AppTitle "Krischans Blue Marble"

Graphics3D 1280,800,32,1

Include "include/milkyway.bb"

Const StarSingleTexture$="textures/star.png"	; single star Texture
Const StarFieldsTexture$="textures/stars.png"	; background starfield texture
Const FogTexture$="textures/milkyway.jpg"		; 360 split panorama nebula texture

Const StarTexScale#=1.5							; texture scaling of background stars
Const PanoSegments%=32							; number of panorama segments (should be at least 8)

Global SUNTEX%,SUN%,SUN0%,SUN1%,SUN2%,SUN3%

Global WIDTH%=GraphicsWidth()
Global HEIGHT%=GraphicsHeight()
Global fader#=1.0

Const PR%=128				; planet colors RED
Const PG%=192				; planet colors GREEN
Const PB%=255				; planet colors BLUE

; media
Global earthtex%=LoadTexture("textures/earth.jpg")
Global nighttex%=LoadTexture("textures/earthnight.jpg")
Global cloudtex%=LoadTexture("textures/clouds.png",2)
Global moontex%=LoadTexture("textures/moon.jpg")
Global atmotex%=LoadTexture("textures/atmo.png")
Global masktex%=LoadTexture("textures/earthmask.png")
Global music%=LoadSound("music/Zeropage - Ambient Flight (Electro Vocal Mix).mp3")

; pivots
Global cpiv%=CreatePivot()
Global mpiv%=CreatePivot(cpiv)

; camera
Global cam%=CreateCamera(cpiv)
PositionEntity cam,-1.5*100.0,0.5*100,1.5*100.0
CameraRange cam,0.1,WIDTH+1

; fade quad
Global fade%=CreateQuad(cam,10,False,False,1+2+16+32,0,0,0,1)
PositionEntity fade,0,0,0
ScaleEntity fade,WIDTH,HEIGHT,1
MoveEntity fade,0,0,WIDTH
EntityOrder fade,-100

; light
Global light%=CreateLight(2)
PositionEntity light,0,0,-20
LightRange light,20
AmbientLight 1,1,1

; earth
Global earth%=CreateSphere(64)
EntityPickMode earth,1,1
EntityRadius earth,1.5
EntityTexture earth,masktex,0,0
EntityTexture earth,nighttex,0,1
EntityTexture earth,earthtex,0,2
TextureBlend masktex,1
TextureBlend nighttex,3
TextureBlend earthtex,2

; cloudsphere
Global clouds%=CreateSphere(64,earth)
ScaleEntity clouds,1.005,1.005,1.005
EntityTexture clouds,cloudtex,0,0
TextureBlend cloudtex,5

; atmosphere shaders
Global atmo%=CreatePlanetShader(16,16,PR,PG,PB,earth)
Global glow1%=CreateGlow(0.95,1.05,360,1+2+32,3,PR,PG,PB,0.2,0,0,0,0)
Global glow2%=InitGlow(earth,0.5,0.5)
ScaleMesh atmo,1.0/8,1.0/8,1.0/8
EntityTexture atmo,atmotex
EntityBlend atmo,3
EntityFX atmo,2+16
ScaleTexture atmotex,-1,1
EntityOrder glow1,1
EntityOrder glow2,-2
EntityOrder atmo,-1

; moon
Global moon%=CreateSphere(32,mpiv)
EntityPickMode moon, 1, 1 
EntityRadius moon,0.25
ScaleEntity moon,.277,.277,.277
MoveEntity moon,3,0,-3
EntityTexture moon,moontex

; milkyway
Global milkyway%=InitMilkyway(500,0.2,10,40,500,0,0,0,0,255,192,128,0.5,0,0,0,0)

; sun
InitSun()
PositionEntity SUN,0,0,-20

Global timer%=CreateTimer(60)

LoopSound music
PlaySound music

MoveMouse WIDTH/2,HEIGHT/2

While Not KeyHit(1)
	
	Local s#,x#,y#,wait%,cl%,sh%,d#
	
	; fade screen
	If fader>0 Then
		fader=fader-0.005
		UpdateMesh(fade,0,0,0,fader)
	EndIf
	
	; SPACE = freeze
	If KeyHit(57) Then wait=1-wait
	
	; ENTER = save screenshot
	If KeyHit(28) Then SaveBuffer BackBuffer(),MilliSecs()+".bmp"
	
	; C = show/hide clouds
	If KeyHit(46) Then
		cl=1-cl
		If cl=1 Then HideEntity clouds Else ShowEntity clouds
	EndIf
	
	; S = show/hide "shaders"
	If KeyHit(31) Then
		sh=1-sh
		If sh=1 Then
			HideEntity atmo
			HideEntity glow1
			HideEntity glow2
		Else
			ShowEntity atmo
			ShowEntity glow1
			ShowEntity glow2
		EndIf
	EndIf
	
	; only move if not waiting
	If wait=0 Then
		
		s=s+0.33 Mod 360.0
		x=x+1 Mod 180.0
		y=y+0.5 Mod 180.0
		
		; rotations
		PositionTexture masktex,-Normalize(s,0,360,0,1),0
		RotateEntity earth,0,s,0
		TurnEntity clouds,0,0.02,0
		TurnEntity cpiv,0,.5,0
		TurnEntity mpiv,0,0.63,0
		
		; initial move
		d=1-(1.33/Exp(EntityDistance(cam,earth)*0.1))
		
		; moves
		MoveEntity cam,(Sin(y)/150.0)+d,Cos(x*0.25)/180.0,(-Sin(y)/150.0)+d
		
		
	EndIf
		
	; milkway holds position
	PositionEntity milkyway,EntityX(cam,1),EntityY(cam,1),EntityZ(cam,1)
	
	; points
	PointEntity SUN,cam
	PointEntity atmo,cam
	PointEntity cam,earth
	
	; glows
	UpdateGlow(glow1,cam,earth,1.0)
	UpdateGlow(atmo,cam,earth,1.04)
	
	; moon order for glow
	If EntityVisible(cam,moon) Then EntityOrder moon,-10 Else EntityOrder moon,0
	
	UpdateWorld
	RenderWorld
	
	WaitTimer timer
	
	Flip 1
	
Wend

End

Function Normalize#(value#=128.0,value_min#=0.0,value_max#=255.0,norm_min#=0.0,norm_max#=1.0)
	
	Return ((value#-value_min#)/(value_max#-value_min#))*(norm_max#-norm_min#)+norm_min#
	
End Function

Function InitGlow(parent%,shininess#,glowalpha#)
	
	Local mesh%=CreateSphere(64,parent)
	Local tex%=CreateFakeShaderTexture()
	
	EntityBlend mesh,3
	EntityTexture mesh,tex,0,1
	
	EntityFX mesh,2
	EntityShininess mesh,shininess
	
	UpdateMesh(mesh,64,96,255,glowalpha)
	
	Return mesh
	
End Function

Function CreateFakeShaderTexture()
	
	Local tex%=CreateTexture(512,512,64)
	Local tb%=TextureBuffer(tex)
	
	Local x%,y%,i#,j%,col%,rgb%
	
	SetBuffer tb
	LockBuffer tb
	
	For x=0 To 511
		
		For y=0 To 511
			
			rgb=255*$1000000+255*$10000+255*$100+255
			WritePixelFast x,y,rgb,tb
			
		Next
		
	Next
	
	For j=0 To 255
		
		col=j*1.0/Exp((255-j)*0.02)
		
		If col>255 Then col=255
		If col<0 Then col=0
		
		rgb=col*$1000000+col*$10000+col*$100+col
		
		For i=0 To 360 Step 0.1
			
			WritePixelFast 256+(Sin(i)*j),256+(Cos(i)*j),rgb,tb
			
		Next
		
	Next
	
	UnlockBuffer tb
	SetBuffer BackBuffer()
	
	Return tex
	
End Function

Function UpdateMesh(mesh%,r%=255,g%=255,b%=255,a#=1.0)
	
	Local v%
	Local surf%=GetSurface(mesh,1)
	
	For v=0 To CountVertices(surf)-1
		VertexColor surf,v,r,g,b,a
	Next
	
End Function

Function CreateGlow(radius1#=1.0,radius2#=2.0,segments%=360,fx%=0,blend%=0,r1%=255,g1%=255,b1%=255,al1#=0.0,r2%=0,g2%=0,b2%=0,al2#=1.0)
	
	Local a1#,a2#,a3#,a4#,angle%,v0%,v1%,v2%,v3%
	Local mesh=CreateMesh()
	Local surf=CreateSurface(mesh)
	
	If segments>360 Then segments=360
	
	For angle=1 To segments
		
		a1=angle*360.0/segments
		a2=angle*360.0/segments +360.0/segments
		a3=angle*360.0/segments +180.0/segments
		a4=angle*360.0/segments -180.0/segments
		
		v0=AddVertex(surf,radius1*Cos(a1),radius1*Sin(a1),0,0,0)
		v1=AddVertex(surf,radius1*Cos(a2),radius1*Sin(a2),0,0,0)
		v2=AddVertex(surf,radius2*Cos(a3),radius2*Sin(a3),0,1,1)
		v3=AddVertex(surf,radius2*Cos(a4),radius2*Sin(a4),0,0,1)
		
		VertexColor surf,v0,r1,g1,b1,al1
		VertexColor surf,v1,r1,g1,b1,al1
		VertexColor surf,v2,r2,g2,b2,al2
		VertexColor surf,v3,r2,g2,b2,al2
		
		AddTriangle surf,v2,v1,v0
		AddTriangle surf,v0,v3,v2
		
	Next
	
	If fx>0 Then EntityFX mesh,fx
	If blend>0 Then EntityBlend mesh,blend
	
	Return mesh
	
End Function

Function UpdateGlow(mesh%,cam%,parent%,scale#=1.0)
	
	Local radius#,distance#
	Local c1#,a1#,q1#,p1#,h1#,alpha1#,beta1#,gamma1#,alpha2#,b2#,c2#
	
	PointEntity mesh,cam
	
	radius=1
	distance=EntityDistance(cam,parent)
	
	c1=distance
	a1=radius
	q1=a1^2/c1
	p1=c1-q1
	h1=Sqr(p1*q1)
	gamma1=90
	alpha1=ATan(h1/p1)
	beta1=gamma1-alpha1
	
	alpha2=90-(90-beta1)
	b2=a1/Tan(alpha2)
	c2=(Sqr(a1^2+b2^2))/radius*scale
	
	ScaleEntity mesh,c2,c2,c2
	
End Function

Function CreatePlanetShader(xs#=10.0,ys#=10.0,r%=255,g%=255,b%=255,parent=0) 
	
	Local mesh%=CreateMesh(parent)
	Local surf%=CreateSurface(mesh)
	Local v[3]
	
	Local x%,y%,z1#,z2#,z3#,z4#
	
	For x=0 To xs-1
		
		For y=0 To ys-1
			
			z1=-(Abs(x-xs/2.0)+Abs(y-ys/2.0))
			z2=-(Abs(x-xs/2.0)+Abs(y+1-ys/2.0))
			z3=-(Abs(x+1-xs/2.0)+Abs(y+1-ys/2.0))
			z4=-(Abs(x+1-xs/2.0)+Abs(y-ys/2.0))
			
			v[0] = AddVertex(surf,x-xs/2.0,-y+ys/2.0,0,x*1.0/xs,y*1.0/ys)
			v[1] = AddVertex(surf,x-xs/2.0,-y+ys/2.0-1,0,x*1.0/xs,(y+1)*1.0/ys)
			v[2] = AddVertex(surf,x-xs/2.0+1,-y+ys/2.0-1,0,(x+1)*1.0/xs,(y+1)*1.0/ys)
			v[3] = AddVertex(surf,x-xs/2.0+1,-y+ys/2.0,0,(x+1)*1.0/xs,y*1.0/ys)
			
		    AddTriangle(surf,v[3],v[0],v[1])
			AddTriangle(surf,v[2],v[3],v[1])
			
			VertexColor surf,v[0],r,g,b
			VertexColor surf,v[1],r,g,b
			VertexColor surf,v[2],r,g,b
			VertexColor surf,v[3],r,g,b
			
			VertexNormal surf,v[0],(x-xs/2)/xs*2,(-y+ys/2.0)/ys*2,1
			VertexNormal surf,v[1],(x-xs/2)/xs*2,(-y+ys/2.0-1)/ys*2,1
			VertexNormal surf,v[2],(x-xs/2+1)/xs*2,(-y+ys/2.0-1)/ys*2,1
			VertexNormal surf,v[3],(x-xs/2+1)/xs*2,(-y+ys/2.0)/ys*2,1
			
		Next
		
	Next
	
	Return mesh
	
End Function 

Function InitSun(size#=1.0)
	
	SUNTEX=CreateSunTexture()
	SUN=CreatePivot()
	SUN0=CreateQuad(SUN,size*1.0,SUNTEX,3,1+8,255,255,255,1.00)
	SUN1=CreateQuad(SUN,size*1.5,SUNTEX,3,1+8,255,192,128,1.00)
	SUN2=CreateQuad(SUN,size*3.0,SUNTEX,3,1+8,255,255,224,0.75)
	SUN3=CreateQuad(SUN,size*15.0,SUNTEX,3,1+8,255,255,224,0.50)
	
	
End Function

Function CreateSunTexture()
	
	Local tex%=CreateTexture(512,512,3)
	Local tb%=TextureBuffer(tex)
	
	Local i#,j%,col%,rgb%
	
	SetBuffer tb
	LockBuffer tb
	
	For j=0 To 255
		
		col=255-j
		If col>255 Then col=255
		rgb=col*$1000000+col*$10000+col*$100+col
		
		For i=0 To 360 Step 0.1
			
			WritePixelFast 256+(Sin(i)*j),256+(Cos(i)*j),rgb,tb
			
		Next
		
	Next
	
	UnlockBuffer tb
	SetBuffer BackBuffer()
	
	Return tex
	
End Function

Function CreateQuad(parent%=False,scale#=1.0,tex%=False,blend%=False,fx%=False,r%=255,g%=255,b%=255,a#=1.0)
	
	Local mesh%=CreateMesh()
	Local surf%=CreateSurface(mesh)
	
	Local v0%=AddVertex(surf, 1, 1,0,0,0)
	Local v1%=AddVertex(surf,-1, 1,0,1,0)
	Local v2%=AddVertex(surf,-1,-1,0,1,1)
	Local v3%=AddVertex(surf, 1,-1,0,0,1)
	
	AddTriangle surf,v0,v1,v2
	AddTriangle surf,v0,v2,v3
	
	If parent Then EntityParent mesh,parent
	If fx Then EntityFX mesh,fx
	If tex Then EntityTexture mesh,tex
	If blend Then EntityBlend mesh,blend
	
	EntityColor mesh,r,g,b
	EntityAlpha mesh,a
	
	VertexColor surf,v0,r,g,b,a
	VertexColor surf,v1,r,g,b,a
	VertexColor surf,v2,r,g,b,a
	VertexColor surf,v3,r,g,b,a
	
	ScaleEntity mesh,scale,scale,scale
	
	Return mesh
	
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D